package lcsb.mapviewer.converter.model.sbgnml;

import static org.junit.Assert.*;

import java.awt.geom.Point2D;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.junit.Test;

import lcsb.mapviewer.common.Configuration;
import lcsb.mapviewer.converter.Converter;
import lcsb.mapviewer.converter.ConverterParams;
import lcsb.mapviewer.model.map.compartment.Compartment;
import lcsb.mapviewer.model.map.model.Model;
import lcsb.mapviewer.model.map.reaction.*;
import lcsb.mapviewer.model.map.species.Complex;
import lcsb.mapviewer.model.map.species.GenericProtein;
import lcsb.mapviewer.model.map.species.field.Residue;

public class SbgnmlXmlParserTest2 extends SbgnmlTestFunctions {

  Logger logger = LogManager.getLogger(SbgnmlXmlParserTest2.class.getName());

  @Test
  public void testAdjustModificationCoordinates() throws Exception {
    SbgnmlXmlParser parser = new SbgnmlXmlParser();
    GenericProtein protein = new GenericProtein("id");
    protein.setWidth(50);
    protein.setHeight(30);
    protein.setX(200);
    protein.setX(300);
    Residue mr = new Residue();
    Point2D position = new Point2D.Double(100, 20);
    mr.setPosition(position);
    protein.addResidue(mr);
    parser.adjustModificationCoordinates(protein);

    assertTrue(mr.getPosition().distance(position) > Configuration.EPSILON);

  }

  @Test
  public void createModelWithCompartmentsTest() throws Exception {
    Converter converter = new SbgnmlXmlConverter();

    Model model = converter
        .createModel(new ConverterParams()
            .filename("testFiles/sbgnmlParserTestFiles/sbgnmlFiles/elements_inside_compartment.xml"));
    Complex complexInsideCompartment = model.getElementByElementId("csa1830");
    assertNotNull("Complex inside compartment has undefined compartment", complexInsideCompartment.getCompartment());
    Complex complexOutsideCompartment = model.getElementByElementId("csa1831");
    assertNull("Complex outside compartment has not null compartment", complexOutsideCompartment.getCompartment());
    Compartment compartment = model.getElementByElementId("ca107");
    assertNull("Top compartment has not null compartment", compartment.getCompartment());
  }

  @Test
  public void createModelWithZIndex() throws Exception {
    Converter converter = new SbgnmlXmlConverter();

    Model model = converter
        .createModel(new ConverterParams()
            .filename("testFiles/sbgnmlParserTestFiles/sbgnmlFiles/elements_inside_compartment.xml"));
    Complex complex = model.getElementByElementId("csa1830");
    assertNotNull(complex.getZ());
  }

  @Test
  public void testCoordinatesOfOperators() throws Exception {
    Converter converter = new SbgnmlXmlConverter();
    Model model = converter
        .createModel(new ConverterParams().filename("testFiles/sbgnmlParserTestFiles/sbgnmlFiles/clone-marker.sbgn"));

    Reaction reaction = model.getReactionByReactionId("glyph9");

    Reactant r1 = reaction.getReactants().get(0);
    Reactant r2 = reaction.getReactants().get(1);

    Product p1 = reaction.getProducts().get(0);
    Product p2 = reaction.getProducts().get(1);

    NodeOperator inputOperator = null;
    NodeOperator outputOperator = null;
    for (NodeOperator operator : reaction.getOperators()) {
      if (operator.isProductOperator()) {
        outputOperator = operator;
      } else {
        inputOperator = operator;
      }
    }

    assertEquals(r1.getLine().getEndPoint(), inputOperator.getLine().getBeginPoint());
    assertEquals(r2.getLine().getEndPoint(), inputOperator.getLine().getBeginPoint());

    assertEquals(p1.getLine().getBeginPoint(), outputOperator.getLine().getBeginPoint());
    assertEquals(p2.getLine().getBeginPoint(), outputOperator.getLine().getBeginPoint());
  }

}
