package lcsb.mapviewer.converter.graphics.bioEntity.element.species;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;

import java.awt.*;
import java.awt.image.BufferedImage;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.junit.*;

import lcsb.mapviewer.commands.ColorExtractor;
import lcsb.mapviewer.converter.graphics.ConverterParams;
import lcsb.mapviewer.converter.graphics.GraphicsTestFunctions;
import lcsb.mapviewer.model.map.species.Complex;

public class ComplexConverterTest extends GraphicsTestFunctions {
  Logger logger = LogManager.getLogger(ComplexConverterTest.class);

  ColorExtractor colorExtractor = new ColorExtractor(Color.RED, Color.GREEN, Color.BLUE);

  @Before
  public void setUp() throws Exception {
  }

  @After
  public void tearDown() throws Exception {
  }

  @Test
  public void testDrawText() throws Exception {
    int size = 200;
    BufferedImage bi = new BufferedImage(size, size, BufferedImage.TYPE_INT_ARGB);
    Graphics2D graphics = bi.createGraphics();
    ComplexConverter rc = new ComplexConverter(colorExtractor);

    Complex alias = createComplex();
    rc.drawText(alias, graphics, new ConverterParams());

    boolean onlyWhite = true;
    for (int x = 0; x < size; x++) {
      for (int y = 0; y < size; y++) {
        if (bi.getRGB(x, y) != 0) {
          onlyWhite = false;
        }
      }
    }
    assertFalse(onlyWhite);
  }

  @Test
  public void testDrawTooBigText() throws Exception {
    int size = 200;
    double scale = 100;
    BufferedImage bi = new BufferedImage(size, size, BufferedImage.TYPE_INT_ARGB);
    Graphics2D graphics = bi.createGraphics();
    Color color = graphics.getColor();
    graphics.scale(1.0 / scale, 1.0 / scale);
    ComplexConverter rc = new ComplexConverter(colorExtractor);

    Complex alias = createComplex();
    rc.drawText(alias, graphics, new ConverterParams());

    assertEquals(color, graphics.getColor());
  }

  @Override
  protected Complex createComplex() {
    Complex protein = new Complex("id");
    protein.setName("NAME_OF_THE_ELEMENT");
    protein.setX(10);
    protein.setY(20);
    protein.setWidth(100);
    protein.setHeight(80);
    protein.setColor(Color.WHITE);

    return protein;
  }
}
