"use strict";

/* exported logger */

var AbstractGuiElement = require('../AbstractGuiElement');

// noinspection JSUnusedLocalSymbols
var logger = require('../../logger');

/**
 *
 * @param {Object} params
 * @param {HTMLElement} params.element
 * @param {CustomMap} [params.customMap]
 * @param {Configuration} params.configuration
 * @param {Project} [params.project]
 * @param {ServerConnector} [params.serverConnector]
 *
 * @constructor
 * @extends AbstractGuiElement
 */
function AbstractAnnotatorsDialog(params) {
  AbstractGuiElement.call(this, params);
}

AbstractAnnotatorsDialog.prototype = Object.create(AbstractGuiElement.prototype);
AbstractAnnotatorsDialog.prototype.constructor = AbstractAnnotatorsDialog;

/**
 *
 * @param {BioEntityType} elementType
 * @param {boolean} [includeChildren=false]
 *
 * @returns {BioEntityType[]}
 */
AbstractAnnotatorsDialog.prototype.getAllChildrenTypesIfNeeded = function (elementType, includeChildren) {
  var result = [elementType];
  if (includeChildren) {
    var queue = [elementType];
    var elementTypes = this.getConfiguration().getElementTypes();
    while (queue.length > 0) {
      var type = queue.shift();
      for (var i = 0; i < elementTypes.length; i++) {
        if (type.className === elementTypes[i].parentClass) {
          queue.push(elementTypes[i]);
          result.push(elementTypes[i]);
        }
      }
    }
  }
  return result;
};

module.exports = AbstractAnnotatorsDialog;
