package lcsb.mapviewer.wikipathway.XML;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

import java.awt.geom.Point2D;
import java.io.ByteArrayInputStream;
import java.nio.charset.StandardCharsets;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.junit.*;

import lcsb.mapviewer.common.Pair;
import lcsb.mapviewer.model.graphics.PolylineData;
import lcsb.mapviewer.model.map.model.*;
import lcsb.mapviewer.model.map.species.GenericProtein;
import lcsb.mapviewer.wikipathway.WikipathwaysTestFunctions;

public class ModelContructorTest extends WikipathwaysTestFunctions {
  Logger logger = LogManager.getLogger(ModelContructorTest.class);

  ModelComparator comparator = new ModelComparator();

  private int elementCounter = 0;

  @Before
  public void setUp() throws Exception {
  }

  @After
  public void tearDown() throws Exception {
  }

  @Test
  public void testSplitPolyline() {
    Point2D p1 = new Point2D.Double(1455.5988605049254, 562.0578193221613);
    Point2D p2 = new Point2D.Double(1455.5988605049254, 562.0578193221613);
    PolylineData pd = new PolylineData(p1, p2);

    ModelContructor mc = new ModelContructor();
    Pair<PolylineData, PolylineData> result = mc.splitPolyline(pd);
    assertNotNull(result);
    assertNotNull(result.getLeft());
    assertNotNull(result.getRight());
    assertEquals(2, result.getLeft().getPoints().size());
    assertEquals(2, result.getRight().getPoints().size());
  }

  @Test
  public void testZIndexForElement() throws Exception {
    ModelToGPML parser = new ModelToGPML();
    Model model = createEmptyModel();
    model.addElement(createProtein());
    String xml = parser.getGPML(model);
    assertNotNull(xml);

    Model model2 = new GPMLToModel().getModel(new ByteArrayInputStream(xml.getBytes(StandardCharsets.UTF_8)));
    assertEquals(0, comparator.compare(model2, model));
  }

  private Model createEmptyModel() {
    Model result = new ModelFullIndexed(null);
    result.setWidth(640);
    result.setHeight(480);
    return result;
  }

  private GenericProtein createProtein() {
    GenericProtein result = new GenericProtein("s" + (elementCounter++));
    result.setX(100);
    result.setY(20);
    result.setWidth(100);
    result.setHeight(30);
    result.setZ(7890);
    return result;
  }

}
